package com.nisum.ascend.inventory.controller;

import com.nisum.ascend.inventory.exception.ResourceAlreadyExistsException;
import com.nisum.ascend.inventory.model.Product;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import com.nisum.ascend.inventory.service.ProductService;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import reactor.core.publisher.Mono;
import reactor.core.publisher.Flux;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.reactive.function.server.ServerResponse;
import reactor.core.publisher.Mono;
import com.nisum.ascend.inventory.service.ProductService;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

@CrossOrigin(origins = "*")
@RestController
@RequestMapping("/api/products")
public class ProductController {
    @Autowired
    ProductService productService;

    @DeleteMapping(value = "/{sku}")
    public Mono<ResponseEntity<Void>> deleteProduct(@PathVariable String sku) {
        return productService.removeProductBySku(sku)
                .map(res -> ResponseEntity.ok().<Void>build())
                .defaultIfEmpty(ResponseEntity.notFound().build());
    }

    @GetMapping("/{sku}")
    public ResponseEntity<Mono<Product>> getProductBySku(@PathVariable String sku) {
        Mono<Product> monoProd = productService.getProductBySku(sku);
        HttpStatus status = monoProd != null ? HttpStatus.OK : HttpStatus.NOT_FOUND;
        return new ResponseEntity<>(monoProd, status);
    }

    @GetMapping()
    public ResponseEntity<Flux<Product>> getAllProducts() {
        return ResponseEntity.ok(productService.findAllProducts());
    }

    @PutMapping("/{sku}")
    public Mono<ResponseEntity<Product>> updateProduct(@PathVariable String sku, @RequestBody Product product){
        return productService.updateProductBySku(sku, product)
                .map(updatedProduct -> ResponseEntity.ok(updatedProduct))
                .defaultIfEmpty(ResponseEntity.badRequest().build());
    }

    @PostMapping("")
    public Mono<Product> postProduct(@RequestBody Product product) {
        return productService.createProduct(product).onErrorMap(throwable -> new ResourceAlreadyExistsException());
    }
}